# Check for binary relocation support.
# Written by Hongli Lai
# http://autopackage.org/

AC_DEFUN([AM_BINRELOC],
[
	AC_ARG_ENABLE(binreloc,
		[  --enable-binreloc       compile with binary relocation support
                          (default=enable when available)],
		enable_binreloc=$enableval,enable_binreloc=auto)

	AC_ARG_ENABLE(binreloc-threads,
		[  --enable-binreloc-threads      compile binary relocation with threads support
	                         (default=yes)],
		enable_binreloc_threads=$enableval,enable_binreloc_threads=yes)

	BINRELOC_CFLAGS=
	BINRELOC_LIBS=
	if test "x$enable_binreloc" = "xauto"; then
		AC_CHECK_FILE([/proc/self/maps])
		AC_CACHE_CHECK([whether everything is installed to the same prefix],
			       [br_cv_valid_prefixes], [
				if test "$bindir" = '${exec_prefix}/bin' -a "$sbindir" = '${exec_prefix}/sbin' -a \
					"$datadir" = '${prefix}/share' -a "$libdir" = '${exec_prefix}/lib' -a \
					"$libexecdir" = '${exec_prefix}/libexec' -a "$sysconfdir" = '${prefix}/etc'
				then
					br_cv_valid_prefixes=yes
				else
					br_cv_valid_prefixes=no
				fi
				])
	fi
	AC_CACHE_CHECK([whether binary relocation support should be enabled],
		       [br_cv_binreloc],
		       [if test "x$enable_binreloc" = "xyes"; then
		       	       br_cv_binreloc=yes
		       elif test "x$enable_binreloc" = "xauto"; then
			       if test "x$br_cv_valid_prefixes" = "xyes" -a \
			       	       "x$ac_cv_file__proc_self_maps" = "xyes"; then
				       br_cv_binreloc=yes
			       else
				       br_cv_binreloc=no
			       fi
		       else
			       br_cv_binreloc=no
		       fi])

	if test "x$br_cv_binreloc" = "xyes"; then
		BINRELOC_CFLAGS="-DENABLE_BINRELOC"
		AC_DEFINE(ENABLE_BINRELOC,,[Use binary relocation?])
		if test "x$enable_binreloc_threads" = "xyes"; then
			AC_CHECK_LIB([pthread], [pthread_getspecific])
		fi

		AC_CACHE_CHECK([whether binary relocation should use threads],
			       [br_cv_binreloc_threads],
			       [if test "x$enable_binreloc_threads" = "xyes"; then
					if test "x$ac_cv_lib_pthread_pthread_getspecific" = "xyes"; then
						br_cv_binreloc_threads=yes
					else
						br_cv_binreloc_threads=no
					fi
			        else
					br_cv_binreloc_threads=no
				fi])

		if test "x$br_cv_binreloc_threads" = "xyes"; then
			BINRELOC_LIBS="-lpthread"
			AC_DEFINE(BR_PTHREAD,1,[Include pthread support for binary relocation?])
		else
			BINRELOC_CFLAGS="$BINRELOC_CFLAGS -DBR_PTHREADS=0"
			AC_DEFINE(BR_PTHREAD,0,[Include pthread support for binary relocation?])
		fi
	fi
	AC_SUBST(BINRELOC_CFLAGS)
	AC_SUBST(BINRELOC_LIBS)
])
dnl Taken from vips
dnl From FIND_MOTIF and ACX_PTHREAD, without much understanding
dnl
dnl FIND_GUICHAN[ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]]
dnl ------------------------------------------------
dnl
dnl Find GUICHAN libraries and headers
dnl
dnl Put compile stuff in GUICHAN_INCLUDES
dnl Put link stuff in GUICHAN_LIBS
dnl
dnl Default ACTION-IF-FOUND defines HAVE_GUICHAN
dnl
AC_DEFUN([FIND_GUICHAN], [

GUICHAN_INCLUDES=""
GUICHAN_LIBS=""

AC_ARG_WITH(guichan, 
[  --without-guichan		do not use libguichan])
# Treat --without-guichan like --without-guichan-includes --without-guichan-libraries.
if test "$with_guichan" = "no"; then
	GUICHAN_INCLUDES=no
	GUICHAN_LIBS=no
fi

AC_ARG_WITH(guichan-includes,
[  --with-guichan-includes=DIR	GUICHAN include files are in DIR],
GUICHAN_INCLUDES="-I$withval")
AC_ARG_WITH(guichan-libraries,
[  --with-guichan-libraries=DIR	GUICHAN libraries are in DIR],
GUICHAN_LIBS="-L$withval -lguichan_sdl -lguichan_opengl -lguichan")

AC_MSG_CHECKING(for GUICHAN)

# Look for Guichan.h 
if test "$GUICHAN_INCLUDES" = ""; then
	Guichan_save_LIBS="$LIBS"

	LIBS="$LIBS -lguichan_sdl -lguichan_opengl -lguichan"

	# Check the standard search path
	AC_TRY_COMPILE([
		#include <guichan.hpp>],[int a],[
		GUICHAN_INCLUDES=""
	], [
		# guichan.h is not in the standard search path.

		# A whole bunch of guesses
		for dir in \
			"${prefix}"/*/include \
			/usr/local/include \
			/usr/*/include \
			/usr/local/*/include /usr/*/include \
			"${prefix}"/include/* \
			/usr/include/* /usr/local/include/* /*/include; do
			if test -f "$dir/guichan.hpp"; then
				GUICHAN_INCLUDES="-I$dir"
				break
			fi
		done

		if test "$GUICHAN_INCLUDES" = ""; then
			GUICHAN_INCLUDES=no
		fi
	])

	LIBS="$Guichan_save_LIBS"
fi

# Now for the libraries
if test "$GUICHAN_LIBS" = ""; then
	Guichan_save_LIBS="$LIBS"
	Guichan_save_INCLUDES="$INCLUDES"

	LIBS=" -lguichan_sdl -lguichan_opengl -lguichan $LIBS"
	INCLUDES="$GUICHAN_INCLUDES $INCLUDES"

	# Try the standard search path first
	AC_TRY_LINK([
		#include <stdio.h>
		#include <guichan.hpp>],[gcnGuichanVersion], [
		GUICHAN_LIBS=" -lguichan_sdl -lguichan_opengl -lguichan"
	], [
		# libGuichan is not in the standard search path.

		# A whole bunch of guesses
		for dir in \
			"${prefix}"/*/lib \
			"${prefix}"/*/lib64 \
			/usr/local/lib \
			/usr/local/lib64 \
			/usr/*/lib \
			/usr/*/lib64 \
			/usr/lib64 \
			"${prefix}"/lib/* /usr/lib/* \
			"${prefix}"/lib64/* /usr/lib64/* \
			/usr/local/lib/* /*/lib; do
			if test -d "$dir" && test "`ls $dir/libguichan.* 2> /dev/null`" != ""; then
				GUICHAN_LIBS="-L$dir  -lguichan_sdl -lguichan_opengl -lguichan"
				break
			fi
		done

		if test "$GUICHAN_LIBS" = ""; then
			GUICHAN_LIBS=no
		fi
	])

	LIBS="$Guichan_save_LIBS"
	INCLUDES="$Guichan_save_INCLUDES"
fi

AC_SUBST(GUICHAN_LIBS)
AC_SUBST(GUICHAN_INCLUDES)

# Print a helpful message
Guichan_libraries_result="$GUICHAN_LIBS"
Guichan_includes_result="$GUICHAN_INCLUDES"

if test x"$Guichan_libraries_result" = x""; then
	Guichan_libraries_result="in default path"
fi
if test x"$Guichan_includes_result" = x""; then
	Guichan_includes_result="in default path"
fi

if test "$Guichan_libraries_result" = "no"; then
	Guichan_libraries_result="(none)"
fi
if test "$Guichan_includes_result" = "no"; then
	Guichan_includes_result="(none)"
fi

AC_MSG_RESULT(
  [libraries $Guichan_libraries_result, headers $Guichan_includes_result])

# Finally, execute ACTION-IF-FOUND/ACTION-IF-NOT-FOUND:
if test "$GUICHAN_INCLUDES" != "no" && test "$GUICHAN_LIBS" != "no"; then
        ifelse([$1],,AC_DEFINE(HAVE_GUICHAN,1,[Define if you have Guichan libraries and header files.]),[$1])
        :
else
	GUICHAN_INCLUDES=""
	GUICHAN_LIBS=""
        $2
fi

AM_CONDITIONAL(HAVE_GUICHAN, test "$GUICHAN_LIBS" != "no")
])dnl
dnl
dnl AM_PATH_LIB3DS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl
AC_DEFUN([AM_PATH_LIB3DS],
[

AC_ARG_WITH(lib3ds-prefix,[  --with-lib3ds-prefix=PFX   Prefix where lib3ds is installed (optional)],
            lib3ds_config_prefix="$withval", lib3ds_config_prefix="")
AC_ARG_WITH(lib3ds-exec-prefix,[  --with-lib3ds-exec-prefix=PFX  Exec prefix where lib3ds is installed (optional)],
            lib3ds_config_exec_prefix="$withval", lib3ds_config_exec_prefix="")

  if test x$lib3ds_config_exec_prefix != x ; then
     lib3ds_config_args="$lib3ds_config_args --exec-prefix=$lib3ds_config_exec_prefix"
     if test x${LIB3DS_CONFIG+set} != xset ; then
        LIB3DS_CONFIG=$lib3ds_config_exec_prefix/bin/lib3ds-config
     fi
  fi
  if test x$lib3ds_config_prefix != x ; then
     lib3ds_config_args="$lib3ds_config_args --prefix=$lib3ds_config_prefix"
     if test x${LIB3DS_CONFIG+set} != xset ; then
        LIB3DS_CONFIG=$lib3ds_config_prefix/bin/lib3ds-config
     fi
  fi

  AC_PATH_PROG(LIB3DS_CONFIG, lib3ds-config, no)
  lib3ds_version_min=$1

  AC_MSG_CHECKING(for Lib3ds - version >= $lib3ds_version_min)
  no_lib3ds=""
  if test "$LIB3DS_CONFIG" = "no" ; then
    no_lib3ds=yes
  else
    LIB3DS_CFLAGS=`$LIB3DS_CONFIG --cflags`
    LIB3DS_LIBS=`$LIB3DS_CONFIG --libs`
    lib3ds_version=`$LIB3DS_CONFIG --version`

    lib3ds_major_version=`echo $lib3ds_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    lib3ds_minor_version=`echo $lib3ds_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    lib3ds_micro_version=`echo $lib3ds_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    lib3ds_major_min=`echo $lib3ds_version_min | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    lib3ds_minor_min=`echo $lib3ds_version_min | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    lib3ds_micro_min=`echo $lib3ds_version_min | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    lib3ds_version_proper=`expr \
        $lib3ds_major_version \> $lib3ds_major_min \| \
        $lib3ds_major_version \= $lib3ds_major_min \& \
        $lib3ds_minor_version \> $lib3ds_minor_min \| \
        $lib3ds_major_version \= $lib3ds_major_min \& \
        $lib3ds_minor_version \= $lib3ds_minor_min \& \
        $lib3ds_micro_version \>= $lib3ds_micro_min `

    if test "$lib3ds_version_proper" = "1" ; then
      AC_MSG_RESULT([$lib3ds_major_version.$lib3ds_minor_version.$lib3ds_micro_version])
    else
      AC_MSG_RESULT(no)
      no_lib3ds=yes
    fi
  fi

  if test "x$no_lib3ds" = x ; then
     ifelse([$2], , :, [$2])     
  else
     LIB3DS_CFLAGS=""
     LIB3DS_LIBS=""
     ifelse([$3], , :, [$3])
  fi

  AC_SUBST(LIB3DS_CFLAGS)
  AC_SUBST(LIB3DS_LIBS)
])



# Configure paths for SDL
# Sam Lantinga 9/21/99
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AM_PATH_SDL([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for SDL, and define SDL_CFLAGS and SDL_LIBS
dnl
AC_DEFUN([AM_PATH_SDL],
[dnl 
dnl Get the cflags and libraries from the sdl-config script
dnl
AC_ARG_WITH(sdl-prefix,[  --with-sdl-prefix=PFX   Prefix where SDL is installed (optional)],
            sdl_prefix="$withval", sdl_prefix="")
AC_ARG_WITH(sdl-exec-prefix,[  --with-sdl-exec-prefix=PFX Exec prefix where SDL is installed (optional)],
            sdl_exec_prefix="$withval", sdl_exec_prefix="")
AC_ARG_ENABLE(sdltest, [  --disable-sdltest       Do not try to compile and run a test SDL program],
		    , enable_sdltest=yes)

  if test x$sdl_exec_prefix != x ; then
    sdl_args="$sdl_args --exec-prefix=$sdl_exec_prefix"
    if test x${SDL_CONFIG+set} != xset ; then
      SDL_CONFIG=$sdl_exec_prefix/bin/sdl-config
    fi
  fi
  if test x$sdl_prefix != x ; then
    sdl_args="$sdl_args --prefix=$sdl_prefix"
    if test x${SDL_CONFIG+set} != xset ; then
      SDL_CONFIG=$sdl_prefix/bin/sdl-config
    fi
  fi

  if test "x$prefix" != xNONE; then
    PATH="$prefix/bin:$prefix/usr/bin:$PATH"
  fi
  AC_PATH_PROG(SDL_CONFIG, sdl-config, no, [$PATH])
  min_sdl_version=ifelse([$1], ,0.11.0,$1)
  AC_MSG_CHECKING(for SDL - version >= $min_sdl_version)
  no_sdl=""
  if test "$SDL_CONFIG" = "no" ; then
    no_sdl=yes
  else
    SDL_CFLAGS=`$SDL_CONFIG $sdlconf_args --cflags`
    SDL_LIBS=`$SDL_CONFIG $sdlconf_args --libs`

    sdl_major_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    sdl_minor_version=`$SDL_CONFIG $sdl_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    sdl_micro_version=`$SDL_CONFIG $sdl_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_sdltest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $SDL_CFLAGS"
      CXXFLAGS="$CXXFLAGS $SDL_CFLAGS"
      LIBS="$LIBS $SDL_LIBS"
dnl
dnl Now check if the installed SDL is sufficiently new. (Also sanity
dnl checks the results of sdl-config to some extent
dnl
      rm -f conf.sdltest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "SDL.h"

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.sdltest");
  */
  { FILE *fp = fopen("conf.sdltest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_sdl_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_sdl_version");
     exit(1);
   }

   if (($sdl_major_version > major) ||
      (($sdl_major_version == major) && ($sdl_minor_version > minor)) ||
      (($sdl_major_version == major) && ($sdl_minor_version == minor) && ($sdl_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'sdl-config --version' returned %d.%d.%d, but the minimum version\n", $sdl_major_version, $sdl_minor_version, $sdl_micro_version);
      printf("*** of SDL required is %d.%d.%d. If sdl-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If sdl-config was wrong, set the environment variable SDL_CONFIG\n");
      printf("*** to point to the correct copy of sdl-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_sdl=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_sdl" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$SDL_CONFIG" = "no" ; then
       echo "*** The sdl-config script installed by SDL could not be found"
       echo "*** If SDL was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the SDL_CONFIG environment variable to the"
       echo "*** full path to sdl-config."
     else
       if test -f conf.sdltest ; then
        :
       else
          echo "*** Could not run SDL test program, checking why..."
          CFLAGS="$CFLAGS $SDL_CFLAGS"
          CXXFLAGS="$CXXFLAGS $SDL_CFLAGS"
          LIBS="$LIBS $SDL_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
#include "SDL.h"

int main(int argc, char *argv[])
{ return 0; }
#undef  main
#define main K_and_R_C_main
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding SDL or finding the wrong"
          echo "*** version of SDL. If it is not finding SDL, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means SDL was incorrectly installed"
          echo "*** or that you have moved SDL since it was installed. In the latter case, you"
          echo "*** may want to edit the sdl-config script: $SDL_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          CXXFLAGS="$ac_save_CXXFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     SDL_CFLAGS=""
     SDL_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(SDL_CFLAGS)
  AC_SUBST(SDL_LIBS)
  rm -f conf.sdltest
])
